@@ -8,6 +8,7 @@
 #include <stdexcept>
 #include <stdlib.h>
 #include <string>
+#include <cstring>
 
 #if BITS_IN_JSAMPLE != 8
 # error "qpdf does not support libjpeg built with BITS_IN_JSAMPLE != 8"
@@ -302,8 +303,9 @@ Pl_DCT::compress(void* cinfo_p, Buffer* b)
     unsigned int width = cinfo->image_width *
         QIntC::to_uint(cinfo->input_components);
     size_t expected_size =
-        cinfo->image_height * cinfo->image_width *
-        QIntC::to_uint(cinfo->input_components);
+        QIntC::to_size(cinfo->image_height) *
+        QIntC::to_size(cinfo->image_width) *
+        QIntC::to_size(cinfo->input_components);
     if (b->getSize() != expected_size)
     {
         throw std::runtime_error(
