@@ -3,7 +3,11 @@
 #include <stdexcept>
 #include <string.h>
 #include <limits.h>
-#include <algorithm>
+
+static int abs_diff(int a, int b)
+{
+    return a > b ? a - b : b - a;
+}
 
 Pl_PNGFilter::Pl_PNGFilter(char const* identifier, Pipeline* next,
 			   action_e action, unsigned int columns,
@@ -41,12 +45,14 @@ Pl_PNGFilter::Pl_PNGFilter(char const* identifier, Pipeline* next,
             "PNGFilter created with invalid columns value");
     }
     this->bytes_per_row = bpr & UINT_MAX;
-    this->buf1 = new unsigned char[this->bytes_per_row + 1];
-    this->buf2 = new unsigned char[this->bytes_per_row + 1];
-    memset(this->buf1, 0, this->bytes_per_row + 1);
-    memset(this->buf2, 0, this->bytes_per_row + 1);
-    this->cur_row = this->buf1;
-    this->prev_row = this->buf2;
+    this->buf1 = PointerHolder<unsigned char>(
+        true, new unsigned char[this->bytes_per_row + 1]);
+    this->buf2 = PointerHolder<unsigned char>(
+        true, new unsigned char[this->bytes_per_row + 1]);
+    memset(this->buf1.getPointer(), 0, this->bytes_per_row + 1);
+    memset(this->buf2.getPointer(), 0, this->bytes_per_row + 1);
+    this->cur_row = this->buf1.getPointer();
+    this->prev_row = this->buf2.getPointer();
 
     // number of bytes per incoming row
     this->incoming = (action == a_encode ?
@@ -56,8 +62,6 @@ Pl_PNGFilter::Pl_PNGFilter(char const* identifier, Pipeline* next,
 
 Pl_PNGFilter::~Pl_PNGFilter()
 {
-    delete [] buf1;
-    delete [] buf2;
 }
 
 void
@@ -77,7 +81,7 @@ Pl_PNGFilter::write(unsigned char* data, size_t len)
 	// Swap rows
 	unsigned char* t = this->prev_row;
 	this->prev_row = this->cur_row;
-	this->cur_row = t ? t : this->buf2;
+	this->cur_row = t ? t : this->buf2.getPointer();
 	memset(this->cur_row, 0, this->bytes_per_row + 1);
 	left = this->incoming;
 	this->pos = 0;
@@ -149,7 +153,7 @@ Pl_PNGFilter::decodeSub()
             left = buffer[i - bpp];
         }
 
-        buffer[i] += left;
+        buffer[i] = static_cast<unsigned char>(buffer[i] + left);
     }
 }
 
@@ -163,7 +167,7 @@ Pl_PNGFilter::decodeUp()
     for (unsigned int i = 0; i < this->bytes_per_row; ++i)
     {
         unsigned char up = above_buffer[i];
-        buffer[i] += up;
+        buffer[i] = static_cast<unsigned char>(buffer[i] + up);
     }
 }
 
@@ -186,7 +190,7 @@ Pl_PNGFilter::decodeAverage()
         }
 
         up = above_buffer[i];
-        buffer[i] += (left+up) / 2;
+        buffer[i] = static_cast<unsigned char>(buffer[i] + (left+up) / 2);
     }
 }
 
@@ -210,7 +214,9 @@ Pl_PNGFilter::decodePaeth()
             upper_left = above_buffer[i - bpp];
         }
 
-        buffer[i] += this->PaethPredictor(left, up, upper_left);
+        buffer[i] = static_cast<unsigned char>(
+            buffer[i] +
+            this->PaethPredictor(left, up, upper_left));
     }
 }
 
@@ -218,9 +224,9 @@ int
 Pl_PNGFilter::PaethPredictor(int a, int b, int c)
 {
     int p = a + b - c;
-    int pa = std::abs(p - a);
-    int pb = std::abs(p - b);
-    int pc = std::abs(p - c);
+    int pa = abs_diff(p, a);
+    int pb = abs_diff(p, b);
+    int pc = abs_diff(p, c);
 
     if (pa <= pb && pa <= pc)
     {
@@ -243,7 +249,8 @@ Pl_PNGFilter::encodeRow()
     {
 	for (unsigned int i = 0; i < this->bytes_per_row; ++i)
 	{
-	    ch = this->cur_row[i] - this->prev_row[i];
+	    ch = static_cast<unsigned char>(
+                this->cur_row[i] - this->prev_row[i]);
 	    getNext()->write(&ch, 1);
 	}
     }
@@ -262,7 +269,7 @@ Pl_PNGFilter::finish()
 	processRow();
     }
     this->prev_row = 0;
-    this->cur_row = buf1;
+    this->cur_row = buf1.getPointer();
     this->pos = 0;
     memset(this->cur_row, 0, this->bytes_per_row + 1);
 
