@@ -314,33 +314,46 @@ QPDFObjectHandle
 QPDFPageObjectHelper::getAttribute(std::string const& name,
                                    bool copy_if_shared)
 {
-    bool inheritable = ((name == "/MediaBox") || (name == "/CropBox") ||
-                        (name == "/Resources") || (name == "/Rotate"));
-
-    QPDFObjectHandle node = this->oh;
-    QPDFObjectHandle result(node.getKey(name));
-    std::set<QPDFObjGen> seen;
+    QPDFObjectHandle result;
+    QPDFObjectHandle dict;
+    bool is_form_xobject = this->oh.isFormXObject();
     bool inherited = false;
-    while (inheritable && result.isNull() && node.hasKey("/Parent"))
+    if (is_form_xobject)
     {
-        seen.insert(node.getObjGen());
-        node = node.getKey("/Parent");
-        if (seen.count(node.getObjGen()))
-        {
-            break;
-        }
+        dict = this->oh.getDict();
+        result = dict.getKey(name);
+    }
+    else
+    {
+        dict = this->oh;
+        bool inheritable = ((name == "/MediaBox") || (name == "/CropBox") ||
+                            (name == "/Resources") || (name == "/Rotate"));
+
+        QPDFObjectHandle node = dict;
         result = node.getKey(name);
-        if (! result.isNull())
+        std::set<QPDFObjGen> seen;
+        while (inheritable && result.isNull() && node.hasKey("/Parent"))
         {
-            QTC::TC("qpdf", "QPDFPageObjectHelper non-trivial inheritance");
-            inherited = true;
+            seen.insert(node.getObjGen());
+            node = node.getKey("/Parent");
+            if (seen.count(node.getObjGen()))
+            {
+                break;
+            }
+            result = node.getKey(name);
+            if (! result.isNull())
+            {
+                QTC::TC("qpdf", "QPDFPageObjectHelper non-trivial inheritance");
+                inherited = true;
+            }
         }
     }
     if (copy_if_shared && (inherited || result.isIndirect()))
     {
-        QTC::TC("qpdf", "QPDFPageObjectHelper copy shared attribute");
+        QTC::TC("qpdf", "QPDFPageObjectHelper copy shared attribute",
+                is_form_xobject ? 0 : 1);
         result = result.shallowCopy();
-        this->oh.replaceKey(name, result);
+        dict.replaceKey(name, result);
     }
     return result;
 }
@@ -373,29 +386,150 @@ QPDFPageObjectHelper::getMediaBox(bool copy_if_shared)
     return getAttribute("/MediaBox", copy_if_shared);
 }
 
+void
+QPDFPageObjectHelper::forEachXObject(
+    bool recursive,
+    std::function<void(QPDFObjectHandle& obj,
+                       QPDFObjectHandle& xobj_dict,
+                       std::string const& key)> action,
+    std::function<bool(QPDFObjectHandle)> selector)
+{
+    QTC::TC("qpdf", "QPDFPageObjectHelper::forEachXObject",
+            recursive
+            ? (this->oh.isFormXObject() ? 0 : 1)
+            : (this->oh.isFormXObject() ? 2 : 3));
+    std::set<QPDFObjGen> seen;
+    std::list<QPDFPageObjectHelper> queue;
+    queue.push_back(*this);
+    while (! queue.empty())
+    {
+        QPDFPageObjectHelper ph = queue.front();
+        queue.pop_front();
+        QPDFObjGen og = ph.oh.getObjGen();
+        if (seen.count(og))
+        {
+            continue;
+        }
+        seen.insert(og);
+        QPDFObjectHandle resources = ph.getAttribute("/Resources", false);
+        if (resources.isDictionary() && resources.hasKey("/XObject"))
+        {
+            QPDFObjectHandle xobj_dict = resources.getKey("/XObject");
+            for (auto const& key: xobj_dict.getKeys())
+            {
+                QPDFObjectHandle obj = xobj_dict.getKey(key);
+                if ((! selector) || selector(obj))
+                {
+                    action(obj, xobj_dict, key);
+                }
+                if (recursive && obj.isFormXObject())
+                {
+                    queue.push_back(QPDFPageObjectHelper(obj));
+                }
+            }
+	}
+    }
+}
+
+void
+QPDFPageObjectHelper::forEachImage(
+    bool recursive,
+    std::function<void(QPDFObjectHandle& obj,
+                       QPDFObjectHandle& xobj_dict,
+                       std::string const& key)> action)
+{
+    forEachXObject(recursive, action,
+                   [](QPDFObjectHandle obj) { return obj.isImage(); });
+}
+
+void
+QPDFPageObjectHelper::forEachFormXObject(
+    bool recursive,
+    std::function<void(QPDFObjectHandle& obj,
+                       QPDFObjectHandle& xobj_dict,
+                       std::string const& key)> action)
+{
+    forEachXObject(recursive, action,
+                   [](QPDFObjectHandle obj) { return obj.isFormXObject(); });
+}
+
 std::map<std::string, QPDFObjectHandle>
 QPDFPageObjectHelper::getPageImages()
 {
-    return this->oh.getPageImages();
+    return getImages();
+}
+
+std::map<std::string, QPDFObjectHandle>
+QPDFPageObjectHelper::getImages()
+{
+    std::map<std::string, QPDFObjectHandle> result;
+    forEachImage(false, [&result](QPDFObjectHandle& obj,
+                                  QPDFObjectHandle&,
+                                  std::string const& key) {
+        result[key] = obj;
+    });
+    return result;
+}
+
+std::map<std::string, QPDFObjectHandle>
+QPDFPageObjectHelper::getFormXObjects()
+{
+    std::map<std::string, QPDFObjectHandle> result;
+    forEachFormXObject(false, [&result](QPDFObjectHandle& obj,
+                                        QPDFObjectHandle&,
+                                        std::string const& key) {
+        result[key] = obj;
+    });
+    return result;
 }
 
 void
 QPDFPageObjectHelper::externalizeInlineImages(size_t min_size)
 {
-    QPDFObjectHandle resources = getAttribute("/Resources", true);
-    // Calling mergeResources also ensures that /XObject becomes
-    // direct and is not shared with other pages.
-    resources.mergeResources(
-        QPDFObjectHandle::parse("<< /XObject << >> >>"));
-    InlineImageTracker iit(this->oh.getOwningQPDF(), min_size, resources);
-    Pl_Buffer b("new page content");
-    filterPageContents(&iit, &b);
-    if (iit.any_images)
+    externalizeInlineImages(min_size, false);
+}
+
+void
+QPDFPageObjectHelper::externalizeInlineImages(size_t min_size, bool shallow)
+{
+    if (shallow)
     {
-        getObjectHandle().replaceKey(
-            "/Contents",
-            QPDFObjectHandle::newStream(
-                this->oh.getOwningQPDF(), b.getBuffer()));
+        QPDFObjectHandle resources = getAttribute("/Resources", true);
+        // Calling mergeResources also ensures that /XObject becomes
+        // direct and is not shared with other pages.
+        resources.mergeResources(
+            QPDFObjectHandle::parse("<< /XObject << >> >>"));
+        InlineImageTracker iit(this->oh.getOwningQPDF(), min_size, resources);
+        Pl_Buffer b("new page content");
+        filterContents(&iit, &b);
+        if (iit.any_images)
+        {
+            if (this->oh.isFormXObject())
+            {
+                this->oh.replaceStreamData(
+                    b.getBuffer(),
+                    QPDFObjectHandle::newNull(),
+                    QPDFObjectHandle::newNull());
+            }
+            else
+            {
+                this->oh.replaceKey(
+                    "/Contents",
+                    QPDFObjectHandle::newStream(
+                        this->oh.getOwningQPDF(), b.getBuffer()));
+            }
+        }
+    }
+    else
+    {
+        externalizeInlineImages(min_size, true);
+        forEachFormXObject(
+            true,
+            [min_size](QPDFObjectHandle& obj,
+                       QPDFObjectHandle&, std::string const&) {
+                QPDFPageObjectHelper(obj).externalizeInlineImages(
+                    min_size, true);
+            });
     }
 }
 
@@ -450,7 +584,21 @@ void
 QPDFPageObjectHelper::parsePageContents(
     QPDFObjectHandle::ParserCallbacks* callbacks)
 {
-    this->oh.parsePageContents(callbacks);
+    parseContents(callbacks);
+}
+
+void
+QPDFPageObjectHelper::parseContents(
+    QPDFObjectHandle::ParserCallbacks* callbacks)
+{
+    if (this->oh.isFormXObject())
+    {
+        this->oh.parseAsContents(callbacks);
+    }
+    else
+    {
+        this->oh.parsePageContents(callbacks);
+    }
 }
 
 void
@@ -458,20 +606,55 @@ QPDFPageObjectHelper::filterPageContents(
     QPDFObjectHandle::TokenFilter* filter,
     Pipeline* next)
 {
-    this->oh.filterPageContents(filter, next);
+    return filterContents(filter, next);
+}
+
+void
+QPDFPageObjectHelper::filterContents(
+    QPDFObjectHandle::TokenFilter* filter,
+    Pipeline* next)
+{
+    if (this->oh.isFormXObject())
+    {
+        this->oh.filterAsContents(filter, next);
+    }
+    else
+    {
+        this->oh.filterPageContents(filter, next);
+    }
 }
 
 void
 QPDFPageObjectHelper::pipePageContents(Pipeline* p)
 {
-    this->oh.pipePageContents(p);
+    pipeContents(p);
+}
+
+void
+QPDFPageObjectHelper::pipeContents(Pipeline* p)
+{
+    if (this->oh.isFormXObject())
+    {
+        this->oh.pipeStreamData(p, 0, qpdf_dl_specialized);
+    }
+    else
+    {
+        this->oh.pipePageContents(p);
+    }
 }
 
 void
 QPDFPageObjectHelper::addContentTokenFilter(
     PointerHolder<QPDFObjectHandle::TokenFilter> token_filter)
 {
-    this->oh.addContentTokenFilter(token_filter);
+    if (this->oh.isFormXObject())
+    {
+        this->oh.addTokenFilter(token_filter);
+    }
+    else
+    {
+        this->oh.addContentTokenFilter(token_filter);
+    }
 }
 
 class NameWatcher: public QPDFObjectHandle::TokenFilter
@@ -508,23 +691,21 @@ NameWatcher::handleToken(QPDFTokenizer::Token const& token)
 
 void
 QPDFPageObjectHelper::removeUnreferencedResourcesHelper(
-    QPDFObjectHandle oh, std::set<QPDFObjGen>& seen,
-    std::function<QPDFObjectHandle()> get_resource,
-    std::function<void(QPDFObjectHandle::TokenFilter*)> filter_content)
+    QPDFPageObjectHelper ph, std::set<QPDFObjGen>& seen)
 {
-    if (seen.count(oh.getObjGen()))
+    if (seen.count(ph.oh.getObjGen()))
     {
         return;
     }
-    seen.insert(oh.getObjGen());
+    seen.insert(ph.oh.getObjGen());
     NameWatcher nw;
     try
     {
-        filter_content(&nw);
+        ph.filterContents(&nw);
     }
     catch (std::exception& e)
     {
-        oh.warnIfPossible(
+        ph.oh.warnIfPossible(
             std::string("Unable to parse content stream: ") + e.what() +
             "; not attempting to remove unreferenced objects from this page");
         return;
@@ -532,7 +713,7 @@ QPDFPageObjectHelper::removeUnreferencedResourcesHelper(
     if (nw.saw_bad)
     {
         QTC::TC("qpdf", "QPDFPageObjectHelper bad token finding names");
-        oh.warnIfPossible(
+        ph.oh.warnIfPossible(
             "Bad token found while scanning content stream; "
             "not attempting to remove unreferenced objects from this page");
         return;
@@ -545,7 +726,7 @@ QPDFPageObjectHelper::removeUnreferencedResourcesHelper(
     std::vector<std::string> to_filter;
     to_filter.push_back("/Font");
     to_filter.push_back("/XObject");
-    QPDFObjectHandle resources = get_resource();
+    QPDFObjectHandle resources = ph.getAttribute("/Resources", true);
     for (std::vector<std::string>::iterator d_iter = to_filter.begin();
          d_iter != to_filter.end(); ++d_iter)
     {
@@ -565,27 +746,11 @@ QPDFPageObjectHelper::removeUnreferencedResourcesHelper(
                 dict.removeKey(*k_iter);
             }
             QPDFObjectHandle resource = dict.getKey(*k_iter);
-            if (resource.isStream() &&
-                resource.getDict().getKey("/Type").isName() &&
-                ("/XObject" == resource.getDict().getKey("/Type").getName()) &&
-                resource.getDict().getKey("/Subtype").isName() &&
-                ("/Form" == resource.getDict().getKey("/Subtype").getName()))
+            if (resource.isFormXObject())
             {
                 QTC::TC("qpdf", "QPDFPageObjectHelper filter form xobject");
                 removeUnreferencedResourcesHelper(
-                    resource.getDict(), seen,
-                    [&resource]() {
-                        auto result = resource.getDict().getKey("/Resources");
-                        if (result.isDictionary())
-                        {
-                            result = result.shallowCopy();
-                            resource.getDict().replaceKey("/Resources", result);
-                        }
-                        return result;
-                    },
-                    [&resource](QPDFObjectHandle::TokenFilter* f) {
-                        resource.filterAsContents(f);
-                    });
+                    QPDFPageObjectHelper(resource), seen);
             }
         }
     }
@@ -595,12 +760,7 @@ void
 QPDFPageObjectHelper::removeUnreferencedResources()
 {
     std::set<QPDFObjGen> seen;
-    removeUnreferencedResourcesHelper(
-        this->oh, seen,
-        [this]() { return this->getAttribute("/Resources", true); },
-        [this](QPDFObjectHandle::TokenFilter* f) {
-            this->filterPageContents(f);
-        });
+    removeUnreferencedResourcesHelper(*this, seen);
 }
 
 QPDFPageObjectHelper
@@ -778,8 +938,8 @@ QPDFPageObjectHelper::placeFormXObject(
     QPDFObjectHandle::Rectangle bbox = bbox_obj.getArrayAsRectangle();
     QPDFObjectHandle::Rectangle T = wmatrix.transformRectangle(bbox);
 
-    // Calculate a scale factor, if needed. If the transformed
-    // rectangle is too big, shrink it. Never expand it.
+    // Calculate a scale factor, if needed. Shrink or expand if needed
+    // and allowed.
     if ((T.urx == T.llx) || (T.ury == T.lly))
     {
         // avoid division by zero
@@ -835,3 +995,135 @@ QPDFPageObjectHelper::placeFormXObject(
         name + " Do\n" +
         "Q\n");
 }
+
+void
+QPDFPageObjectHelper::flattenRotation()
+{
+    QPDF* qpdf = this->oh.getOwningQPDF();
+    if (! qpdf)
+    {
+        throw std::runtime_error(
+            "QPDFPageObjectHelper::flattenRotation"
+            " called with a direct object");
+    }
+
+    auto rotate_oh = this->oh.getKey("/Rotate");
+    int rotate = 0;
+    if (rotate_oh.isInteger())
+    {
+        rotate = rotate_oh.getIntValueAsInt();
+    }
+    if (! ((rotate == 90) || (rotate == 180) || (rotate == 270)))
+    {
+        return;
+    }
+    auto mediabox = this->oh.getKey("/MediaBox");
+    if (! mediabox.isRectangle())
+    {
+        return;
+    }
+    auto media_rect = mediabox.getArrayAsRectangle();
+
+    std::vector<std::string> boxes = {
+        "/MediaBox", "/CropBox", "/BleedBox", "/TrimBox", "/ArtBox",
+    };
+    for (auto const& boxkey: boxes)
+    {
+        auto box = this->oh.getKey(boxkey);
+        if (! box.isRectangle())
+        {
+            continue;
+        }
+        auto rect = box.getArrayAsRectangle();
+        decltype(rect) new_rect;
+
+        // How far are the edges of our rectangle from the edges
+        // of the media box?
+        auto left_x   = rect.llx - media_rect.llx;
+        auto right_x  = media_rect.urx - rect.urx;
+        auto bottom_y = rect.lly - media_rect.lly;
+        auto top_y    = media_rect.ury - rect.ury;
+
+        // Rotating the page 180 degrees does not change
+        // /MediaBox. Rotating 90 or 270 degrees reverses llx and
+        // lly and also reverse urx and ury. For all the other
+        // boxes, we want the corners to be the correct distance
+        // away from the corners of the mediabox.
+        switch (rotate)
+        {
+          case 90:
+            new_rect.llx = media_rect.lly + bottom_y;
+            new_rect.urx = media_rect.ury - top_y;
+            new_rect.lly = media_rect.llx + right_x;
+            new_rect.ury = media_rect.urx - left_x;
+            break;
+
+          case 180:
+            new_rect.llx = media_rect.llx + right_x;
+            new_rect.urx = media_rect.urx - left_x;
+            new_rect.lly = media_rect.lly + top_y;
+            new_rect.ury = media_rect.ury - bottom_y;
+            break;
+
+          case 270:
+            new_rect.llx = media_rect.lly + top_y;
+            new_rect.urx = media_rect.ury - bottom_y;
+            new_rect.lly = media_rect.llx + left_x;
+            new_rect.ury = media_rect.urx - right_x;
+            break;
+
+          default:
+            // ignore
+            break;
+        }
+
+        this->oh.replaceKey(
+            boxkey, QPDFObjectHandle::newFromRectangle(new_rect));
+    }
+
+    // When we rotate the page, pivot about the point 0, 0 and then
+    // translate so the page is visible with the origin point being
+    // the same offset from the lower left corner of the media box.
+    // These calculations have been verified empirically with various
+    // PDF readers.
+    QPDFObjectHandle::Matrix cm;
+    cm.e = 0.0;
+    cm.f = 0.0;
+    switch (rotate)
+    {
+      case 90:
+        cm.b = -1;
+        cm.c = 1;
+        cm.f = media_rect.urx + media_rect.llx;
+        break;
+
+      case 180:
+        cm.a = -1;
+        cm.d = -1;
+        cm.e = media_rect.urx + media_rect.llx;
+        cm.f = media_rect.ury + media_rect.lly;
+        break;
+
+      case 270:
+        cm.b = 1;
+        cm.c = -1;
+        cm.e = media_rect.ury + media_rect.lly;
+        break;
+
+      default:
+        break;
+    }
+    std::string cm_str =
+        std::string("q\n") +
+        QUtil::double_to_string(cm.a, 2) + " " +
+        QUtil::double_to_string(cm.b, 2) + " " +
+        QUtil::double_to_string(cm.c, 2) + " " +
+        QUtil::double_to_string(cm.d, 2) + " " +
+        QUtil::double_to_string(cm.e, 2) + " " +
+        QUtil::double_to_string(cm.f, 2) + " cm\n";
+    this->oh.addPageContents(
+        QPDFObjectHandle::newStream(qpdf, cm_str), true);
+    this->oh.addPageContents(
+        QPDFObjectHandle::newStream(qpdf, "\nQ\n"), false);
+    this->oh.removeKey("/Rotate");
+}
